Method: TaskJuggler::PropertySet#addProperty

Defined in:
lib/taskjuggler/PropertySet.rb

#addProperty(property) ⇒ Object

Add the new PropertyTreeNode object property to the set. The set is indexed by ID. In case an object with the same ID already exists in the set it will be overwritten.

Whenever the set has been extended, the ‘bsi’ and ‘tree’ attributes of the properties are no longer up-to-date. You must call index() before using these attributes.



183
184
185
186
187
188
# File 'lib/taskjuggler/PropertySet.rb', line 183

def addProperty(property)
  # The PropertyTreeNode objects are indexed by ID or hierachical ID
  # depending on the name space setting of this set.
  @propertyMap[property.id] = property
  @properties << property
end