7599
7600
7601
7602
7603
7604
7605
7606
7607
7608
7609
7610
7611
7612
7613
|
# File 'lib/taskjuggler/TjpSyntaxRules.rb', line 7599
def rule_warn
pattern(%w( _warn !logicalExpression ), lambda {
begin
@property.set('warn', @property.get('warn') + [ @val[1] ])
rescue AttributeOverwrite
end
})
doc('warn', <<'EOT'
The warn attribute adds a [[logicalexpression|logical expression]] to the
property. The condition described by the logical expression is checked after
the scheduling and a warning is generated if the condition evaluates to true.
This attribute is primarily intended for testing purposes.
EOT
)
end
|